@@ -32,27 +32,27 @@ Family <- function(ngradient, loss = NULL, risk = NULL,
 
 ### Gaussian (Regression)
 GaussReg <- function()
-    Family(ngradient = function(y, f) y - f,
+    Family(ngradient = function(y, f, w = 1) y - f,
            loss = function(y, f) (y - f)^2,
            offset = weighted.mean,
            name = "Squared Error (Regression)")
 
 ### Gaussian (-1 / 1 Binary Classification)
 GaussClass <- function()
-    Family(ngradient = function(y, f) - 2 * y + 2 * y * f,
+    Family(ngradient = function(y, f, w = 1) - 2 * y + 2 * y * f,
            loss = function(y, f) 1 - 2 * y * f + (y * f)^2,
            name = "Squared Error (Classification)")
 
 ### Laplace
 Laplace <- function()
-    Family(ngradient = function(y, f) sign(y - f),
+    Family(ngradient = function(y, f, w = 1) sign(y - f),
            loss = function(y, f) abs(y - f),
            offset = function(y, w) median(y),
            name = "Absolute Error")
 
 ### Binomial
 Binomial <- function()
-    Family(ngradient = function(y, f) {
+    Family(ngradient = function(y, f, w = 1) {
                exp2yf <- exp(-2 * y * f)
                -(-2 * y * exp2yf) / (log(2) * (1 + exp2yf))
            },
@@ -73,7 +73,7 @@ Binomial <- function()
 
 ### Poisson
 Poisson <- function()
-    Family(ngradient = function(y, f) y - exp(f),
+    Family(ngradient = function(y, f, w = 1) y - exp(f),
            loss = function(y, f) -y*f + exp(f),
            offset = weighted.mean,
            name = "Poisson Likelihood")
@@ -86,7 +86,7 @@ Huber <- function(d = NULL) {
     else
         dtxt <- NULL
     fit <- 0
-    Family(ngradient = function(y, f) {
+    Family(ngradient = function(y, f, w = 1) {
                if (is.null(d)) d <- median(abs(y - fit))
                fit <<- f
                ifelse(abs(y - f) < d, y - f, d * sign(y - f))
@@ -103,7 +103,7 @@ Huber <- function(d = NULL) {
 
 ### Adaboost
 AdaExp <- function()
-    Family(ngradient = function(y, f) y * exp(-y * f),
+    Family(ngradient = function(y, f, w = 1) y * exp(-y * f),
            loss = function(y, f) exp(-y * f),
            offset = function(y, w) {
                p <- weighted.mean(y > 0, w)
@@ -113,26 +113,35 @@ AdaExp <- function()
 
 ### Cox proportional hazards model (partial likelihood)
 CoxPH <- function()
-    Family(ngradient = function(y, f) {
+    Family(ngradient = function(y, f, w) {
                time <- y[,1]
                storage.mode(time) <- "double"
                event <- y[,2]
                storage.mode(event) <- "integer"
+               if (length(w) == 1) w <- rep(w, length(time))
+               storage.mode(w) <- "double"
                if (length(f) == 1)
                    f <- rep(f, length(time))
                storage.mode(f) <- "double"
-               .Call("ngradientCoxPLik", time, event, f)
+               .Call("ngradientCoxPLik", time, event, f, w)
            },
-           loss = function(y, f, w) {
+           loss = plloss <- function(y, f, w) {
                time <- y[,1]
-               storage.mode(time) <- "double"
                event <- y[,2]
-               storage.mode(event) <- "integer"
-               if (length(f) == 1)
-                   f <- rep(f, length(time))
-               storage.mode(f) <- "double"
-               ot <- order(time)
-              .Call("CoxPLik", time[ot], event[ot], f[ot]) * (-1)
+               n <- length(time)
+               if (length(f) == 1) f <- rep(f, n)
+               if (length(w) == 1) w <- rep(w, n)
+               indx <- rep(1:n, w)
+               time <- time[indx]
+               event <- event[indx]
+               ef <- exp(f)[indx]
+               f <- f[indx]
+               n <- length(time)
+               risk <- rep(0, n)
+               for (i in 1:n)
+                   risk[i] <- sum((time >= time[i])*ef)
+               event * (f - log(risk))
            },
-           weights = FALSE, 
+           risk = function(y, f, w = 1) -sum(plloss(y, f, w)),
+           weights = TRUE, 
            name = "Partial Likelihood")
